﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Data;
using System.Globalization;
using System.Reflection;
using System.Text;
using Produmex.Foundation;
using Produmex.Foundation.Collections;
using Produmex.Foundation.Data;
using Produmex.Foundation.Data.Sbo;
using Produmex.Foundation.Data.Sbo.BusinessObjects;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions.Tables;
using Produmex.Foundation.Data.Sbo.DataObjects;
using Produmex.Foundation.Data.Sbo.Utilities;
using Produmex.Foundation.Data.SqlClient;
using Produmex.Foundation.Diagnostics;
using Produmex.Foundation.SlimScreen;
using Produmex.Foundation.SlimScreen.Interfaces;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions.KnownDataSets;
using Produmex.Foundation.Utilities;
using Produmex.Foundation.Workflows;
using Produmex.Foundation.Workflows.Parameters;
using Produmex.Foundation.Wwf.Sbo.LocalServices;
using Produmex.Sbo.Logex.Data.BusinessObjects;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables;
using Produmex.Sbo.Logex.Data.DataObjects;
using Produmex.Sbo.Logex.Data.Providers;
using Produmex.Sbo.Logex.Data.Reports.Definitions;
using Produmex.Sbo.Logex.Korver.BusinessObjects.Definitions.Tables;

// PmxWorkflowExecutionTypes.HOOK_FLOW
namespace WorkflowScript_AfterPickListPackedHookFlow
{
    /// <summary>
    /// This is a hook script, meant to be customized on a "per project" basis
    /// </summary>
    public class WorkflowScript_AfterPickListPackedHookScript : WorkflowInstanceScriptBase
    {



        private static readonly ILog s_log = LogProvider.GetLogger(MethodInfo.GetCurrentMethod().DeclaringType);

        // Input parameters * do not change *
        public ReadOnlyBinder<CultureInfo> DefaultCultureInfo;
        public ReadOnlyBinder<PmxOseCompany> PmxOseCompany;
        public ReadOnlyBinder<int> PickListDocEntry;
        public ReadWriteBinder<bool> DoAutomaticShipping;

        // Output parameters * do not change *
        public ReadWriteBinder<PmxDictionary<string, object>> DeliveryData;

        // Sub-flows
        // <none>

        /// <summary>
        /// Initializes a new instance of the <see cref="WorkflowScript_AfterPickListPackedHookFlow"/> class.
        /// </summary>
        /// <param name="parent">The parent.</param>
        /// <param name="factory">The factory.</param>
        public WorkflowScript_AfterPickListPackedHookScript(WorkflowInstanceBase parent, WorkflowInstanceFactory factory)
            : base(parent, factory)
        {
        }

        #region WorkflowInstanceScriptBase Members

        protected override void Execute()
        {
            // Parameters in scope
            Session session = GetScopeParameter("Session") as Session;
            ISboProviderService sboProviderService = GetScopeParameter("<WwfService>ISboService") as ISboProviderService;
            Produmex.Foundation.Messages.Message msg = null;

            string query = null;
            string msgKey = null;
            string initialErrorKey = null;
            ProdumexError pmxError = null;
            int baseDocKey = 0;
            Collection<ReportParameter> reportParameterCollection = new Collection<ReportParameter>();
            string parameterName = ReportTypeSAPCrystalReportsDefinition.Parameters.DocEntry.NAME;

            bool goBack = false;

            int qtyColli = 0;
            int pickTijd = 0;
            int transportCode = 0;

            double qtyWeight = 0;

            string deviceID = session.DeviceInstanceId;
            string colliType = null;
            string logistiekOpm = null;
            string report = null;

            PmxDictionary<string, int> colliTypes = new PmxDictionary<string, int>();

            DataSet dsColliTypes = null;
            DataSet dsShippingTypes = null;
            DataSet dsSalesDeliveryReports = null;

            string soort = null;
            string totalCollies = null;

            if (DeliveryData.Get() == null)
            {
                DeliveryData.Set(new PmxDictionary<string, object>());
            }

        Step_SelectColliType:

            #region SELECT COLLI TYPE

            if (DeliveryData.Get().ContainsKey(PmxSalesDeliveryTable.Columns.Colli.NAME) &&
                DeliveryData.Get().ContainsKey(PmxSalesDeliveryTable.Columns.Soort.NAME) &&
                !goBack)
            {
                soort = DeliveryData.Get()[PmxSalesDeliveryTable.Columns.Soort.NAME] as string;
                totalCollies = DeliveryData.Get()[PmxSalesDeliveryTable.Columns.Colli.NAME] as string;
                goto Step_EnterWeight;
            }

            query = BuildQuery.GetAllColliTypes();
            dsColliTypes = sboProviderService.RunView(false, null, null, query);

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectEnterStringValueScreen),
                DefaultCultureInfo.Get(), BuildParamCollection(
                "InitialErrorKey", initialErrorKey,
                "TitleKey", "MSG_TITLE_KOR_SELECT_COLLI_TYPES",
                "AllowToGoBack", false,
                "StringValueDS", dsColliTypes));
            initialErrorKey = null;
            msg = WaitForMessage();
            goBack = false;

            if (msg.Name.EndsWith(".StringValueSelected"))
            {
                colliType = ExtractParameter<string>(msg.Parameters, "stringValue");
                goto Step_EnterNumberOfColli;
            }
            else
            {
                //goBack = true;
                return;
            }

            #endregion

        Step_EnterNumberOfColli:

            #region ENTER NUMBER OF COLLI

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterQuantityScreen),
                 DefaultCultureInfo.Get(), BuildParamCollection(
                "InitialErrorKey", initialErrorKey,
                  "TitleKey", "MSG_TITLE_KOR_ENTER_COLLIES",
                "AllowToGoBack", true,
                 "InitialQuantity", 1,
                    "NumberOfDigitsAfterDecimal", 0,
                    "MaximumQuantity", -1,
                    "UOM", ""
                //,
                //"LicensePlateDS", dsColliTypes
                ));

            //clear data
            goBack = false;
            initialErrorKey = null;
            msg = WaitForMessage();
            if (msg.Name.EndsWith(".QuantityEntered"))
            {
                qtyColli = Convert.ToInt32(ExtractParameter<double>(msg.Parameters, "quantity"));
                goto Step_EnterNextColliType;
            }
            else
            {
                goBack = true;
                goto Step_SelectColliType;
            }

            #endregion


        Step_EnterNextColliType:

            #region ENTER NEXT COLLI TYPE

            if (colliTypes.ContainsKey(colliType))
            {
                colliTypes[colliType] = qtyColli;
            }
            else
            {
                colliTypes.Add(colliType, qtyColli);
            }

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
                    DefaultCultureInfo.Get(), BuildParamCollection(
                    "TitleKey", "MSG_TITLE_KOR_NEW_COLLI_TYPE",
                    "MessageKey", "MSG_QUESTION_KOR_NEW_COLLI_TYPE"));
            msg = WaitForMessage();
            goBack = false;
            if (msg.Name.ToLower().EndsWith(".yes"))
            {
                goto Step_SelectColliType;
            }

            goto Step_EnterWeight;

            #endregion


        Step_EnterWeight:

            #region Enter Weight

            if (DeliveryData.Get().ContainsKey(PmxSalesDeliveryTable.Columns.Gewicht.NAME) &&
               !goBack)
            {
                qtyWeight = Convert.ToDouble(DeliveryData.Get()[PmxSalesDeliveryTable.Columns.Gewicht.NAME] as string);
                goto Step_EnterRemarks;
            }

            qtyWeight = 0;
            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterQuantityScreen),
                DefaultCultureInfo.Get(), BuildParamCollection(
                "InitialErrorKey", initialErrorKey,
                    "TitleKey", "MSG_TITLE_KOR_ENTER_WEIGHT",
                    "InitialQuantity", 1,
                    "NumberOfDigitsAfterDecimal", 3,
                    "MaximumQuantity", -1,
                    "UOM", ""
                   ));
            msg = WaitForMessage();
            goBack = false;
            //clear data
            initialErrorKey = null;

            if (msg.Name.EndsWith(".QuantityEntered"))
            {
                qtyWeight = Convert.ToDouble(ExtractParameter<double>(msg.Parameters, "quantity"));
                goto Step_SelectShippingTypes;
            }

            goBack = true;
            goto Step_EnterNextColliType;


            #endregion

        Step_SelectShippingTypes:

            #region SELECT SHIPPING TYPES

            query = BuildQuery.GetAllShippingTypes();
            dsShippingTypes = sboProviderService.RunView(false, null, null, query);

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectEnterStringValueScreen),
            DefaultCultureInfo.Get(), BuildParamCollection(
            "InitialErrorKey", initialErrorKey,
            "TitleKey", "MSG_TITLE_KOR_SELECT_SHIPPING_TYPES",
            "AllowToGoBack", false,
            "StringValueDS", dsShippingTypes));

            initialErrorKey = null;
            msg = WaitForMessage();
            goBack = false;
            //allowGoBack = true;

            if (msg.Name.EndsWith(".StringValueSelected"))
            {
                string shippingType = ExtractParameter<string>(msg.Parameters, "stringValue");

                transportCode = Convertor.ConvertToInt32(dsShippingTypes.Tables[0].Select("stringValue = '" + shippingType + "'")[0][PmxShippingTypeTable.Columns.Code.NAME]);
                goto Step_EnterRemarks;
            }
            else
            {
                goBack = true;
                goto Step_EnterWeight;
            }

            #endregion

        Step_EnterRemarks:

            #region ENTER REMARKS

            if (DeliveryData.Get().ContainsKey(PmxSalesDeliveryTable.Columns.LogistiekOpm.NAME) &&
                !goBack)
            {
                logistiekOpm = DeliveryData.Get()[PmxSalesDeliveryTable.Columns.LogistiekOpm.NAME] as string;
                goto Step_ChooseSalesDeliveryReport;
            }


            // Show screen with list
            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterStringValueScreen),
                DefaultCultureInfo.Get(), BuildParamCollection(
                    "InitialErrorKey", initialErrorKey,
                    "TitleKey", "MSG_TITLE_KOR_ENTER_REMARK",
                    "Information", "MSG_KOR_ENTER_REMARK_DEFAULT",
                    "Parameters", new object[] { },
                    "AllowToGoBack", true,
                    "ForceDataEntry", false,
                    "AllowMultiLine", true,
                    "MinimumNumberOfCharacters", 0
                    ));

            msg = WaitForMessage();
            goBack = false;
            initialErrorKey = null;

            if (msg.Name.EndsWith(".StringEntered"))
            {
                logistiekOpm = string.IsNullOrEmpty(ExtractParameter<string>(msg.Parameters, "stringValue").Trim()) ? string.Empty : ExtractParameter<string>(msg.Parameters, "stringValue").Trim();

                if (logistiekOpm.Length > 50)
                {
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "MessageKey", "MSG_ERROR_KOR_REMARK_TOO_BIG",
                            "IsError", true,
                            "ShowButton", true));
                    WaitForMessage();

                    logistiekOpm = null;
                    goto Step_EnterRemarks;
                }

                goto Step_ChooseSalesDeliveryReport;
            }
            else
            {
                goBack = true;
                goto Step_SelectShippingTypes;
            }

            #endregion

        Step_ChooseSalesDeliveryReport:

            #region CHOOSE SALES DELIVERY REPORT

            if (DeliveryData.Get().ContainsKey(PmxSalesDeliveryTable.Columns.Report.NAME) &&
                    !goBack)
            {
                report = DeliveryData.Get()[PmxSalesDeliveryTable.Columns.Report.NAME] as string;
                //goto Step_EnterPickTime;
				goto Step_FillInDeliveryData;
            }


            query = BuildQuery.GetAllSalesDeliveryReports();
            dsSalesDeliveryReports = sboProviderService.RunView(false, null, null, query);

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectEnterStringValueScreen),
              DefaultCultureInfo.Get(), BuildParamCollection(
              "InitialErrorKey", initialErrorKey,
              "TitleKey", "MSG_TITLE_KOR_SELECT_SALES_REPORTS",
              "AllowToGoBack", false,
              "StringValueDS", dsSalesDeliveryReports));

            msg = WaitForMessage();
            initialErrorKey = null;
            goBack = false;

            if (msg.Name.EndsWith(".StringValueSelected"))
            {
                report = ExtractParameter<string>(msg.Parameters, "stringValue");
                //goto Step_EnterPickTime;
				goto Step_FillInDeliveryData;
            }
            else
            {
                goBack = true;
                goto Step_EnterRemarks;
            }
            #endregion

        Step_EnterPickTime:

            #region ENTER PICK TIME

            if (DeliveryData.Get().ContainsKey(PmxSalesDeliveryTable.Columns.Picktijd.NAME) &&
                !goBack)
            {
                pickTijd = (int) DeliveryData.Get()[PmxSalesDeliveryTable.Columns.Picktijd.NAME];
                goto Step_FillInDeliveryData;
            }


            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterQuantityScreen),
               DefaultCultureInfo.Get(), BuildParamCollection(
               "InitialErrorKey", initialErrorKey,
                   "TitleKey", "MSG_TITLE_KOR_ENTER_PICK_TIME",
                "InitialQuantity", 1,
                   "NumberOfDigitsAfterDecimal", 0,
                   "MaximumQuantity", -1,
                   "UOM", ""
                  ));
            msg = WaitForMessage();
            //clear data
            initialErrorKey = null;

            if (msg.Name.EndsWith(".QuantityEntered"))
            {
                pickTijd = Convert.ToInt32(ExtractParameter<double>(msg.Parameters, "quantity"));
                goto Step_FillInDeliveryData;
            }
            else
            {
                goBack = true;
                goto Step_ChooseSalesDeliveryReport;
            }



            #endregion


        Step_FillInDeliveryData:

            #region FILL IN DELIVERY DATA

            if (totalCollies == null)
            {
                int totalColliesTemp = 0;
                string soortTemp = string.Empty;
                foreach (KeyValuePair<string, int> pair in colliTypes)
                {
                    totalColliesTemp += pair.Value;
                    soortTemp += string.Format("{0} {1},", pair.Value, pair.Key);
                }
                soort = soortTemp.TrimEnd(',');
                totalCollies = totalColliesTemp.ToString();
            }
            PmxDictionary<string, object> deliveryData = new PmxDictionary<string, object>();

            deliveryData.Add(PmxSalesDeliveryTable.Columns.Colli.NAME, totalCollies);
            deliveryData.Add(PmxSalesDeliveryTable.Columns.Gewicht.NAME, qtyWeight.ToString());
            deliveryData.Add(PmxSalesDeliveryTable.Columns.LogistiekOpm.NAME, string.IsNullOrEmpty(logistiekOpm) ? string.Empty : logistiekOpm);
            deliveryData.Add(PmxSalesDeliveryTable.Columns.Picktijd.NAME, pickTijd);
            deliveryData.Add(PmxSalesDeliveryTable.Columns.Soort.NAME, string.IsNullOrEmpty(soort) ? string.Empty : soort);
            deliveryData.Add(PmxSalesDeliveryTable.Columns.Report.NAME, string.IsNullOrEmpty(report) ? string.Empty : report);

            DeliveryData.Set(deliveryData);

            goto Step_UpdateAction;

            #endregion

        Step_UpdateAction:

            #region UPDATE SALES ORDER

            if (transportCode > 0)
            {
                try
                {
                    sboProviderService.InvokeMethodWithDbConnection<object>(false, null, null, new DelegateWithSboConnection<object>(ExecuteUpdateSalesOrder), transportCode, PickListDocEntry.Get());
                }
                catch (Exception ex)
                {
                    s_log.Error("", ex);
                    if (ex is ProdumexException)
                    {
                        // Inform the user with the error
                        session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                            DefaultCultureInfo.Get(), BuildParamCollection(
                                "PmxError", ((ProdumexException)ex).Error,
                                "ShowButton", true));
                        WaitForMessage();
                    }
                    else
                    {
                        // Inform the user with the error
                        session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                            DefaultCultureInfo.Get(), BuildParamCollection(
                                "MessageKey", ExceptionHelper.Format(ex, this.DefaultCultureInfo),
                                "NoTranslationOfMessageKey", true,
                                "IsError", true,
                                "ShowButton", true));
                        WaitForMessage();
                    }

                    //ask user if he wants to try again after error
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "TitleKey", "MSG_TITLE_ERROR_TRY_AGAIN",
                            "MessageKey", "MSG_QUESTION_ERROR_TRY_AGAIN"));
                    msg = WaitForMessage();

                    if (msg.Name.EndsWith(".Yes"))
                    {
                        goto Step_UpdateAction;
                    }
                }
            }

            #endregion

            return;

        }

        /// <summary>
        /// Executes the update sales order.
        /// </summary>
        /// <param name="conn">The connection.</param>
        /// <param name="parameters">The parameters.</param>
        private static object ExecuteUpdateSalesOrder(SboConnection conn, object[] parameters)
        {
            int transportCode = Convertor.ConvertToInt32(parameters[0]);
            int pickListDocEntry = Convertor.ConvertToInt32(parameters[1]);
            //int transportCode = 0;

            //string query = BuildQuery.GetTransportCode(shippingType);
            //using (ISboRecordset rs = SboRecordsetHelper.RunQuery(s_log, query, conn))
            //{
            //    transportCode = rs.GetTypedValue<int>(PmxShippingTypeTable.Columns.Code.NAME);
            //}

            PmxDictionary<int, List<int>> list = new PmxDictionary<int, List<int>>();

            string query = BuildQuery.GetSalesOrderDocEntryAndLineNums(pickListDocEntry);
            using (ISboRecordset rs = SboRecordsetHelper.RunQuery(s_log, query, conn))
            {
                while (!rs.EoF)
                {
                    int orderDocEntry = rs.GetTypedValue<int>(PmxPickListProposalLineTable.Columns.BaseEntry.NAME);
                    int lineNum = rs.GetTypedValue<int>(PmxPickListProposalLineTable.Columns.BaseLine.NAME);

                    if(!list.ContainsKey(orderDocEntry))
                    {
                        list.Add(orderDocEntry, new List<int>());
                    }

                    list[orderDocEntry].Add(lineNum);

                    rs.MoveNext();
                }
            }

            PmxSalesOrderProvider orderProv = new PmxSalesOrderProvider(conn);

            foreach (int orderDocEntry in list.Keys)
            {
                using (PmxSalesOrder order = orderProv.GetBO(orderDocEntry))
                {
                    order.TransportationCode = transportCode;
                    foreach (var line in order.Lines)
                    {
                        if (list[orderDocEntry].Contains(line.LineNum))
                        {
                            line.ShippingMethod = transportCode;
                        }
                    }
                    orderProv.UpdateBO(order, false, false);
                }
            }
            return null;
        }

        #endregion


        #region BUILD QUERY

        /// <summary>
        /// Static class grouping our native queries
        /// </summary>
        internal static class BuildQuery
        {

            /// <summary>
            /// Gets all colli types.
            /// </summary>
            /// <returns></returns>
            public static string GetAllColliTypes()
            {
                StringBuilder strBuilder = new StringBuilder();
                strBuilder.Append("SELECT ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, KorverColliTypeTable.NAME, KorverColliTypeTable.Columns.Name.NAME, StringValueDataSet.Columns.STRING_VALUE.Name);

                strBuilder.Append(" FROM ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(KorverColliTypeTable.NAME));

                return strBuilder.ToString();
            }

            /// <summary>
            /// Gets all colli types.
            /// </summary>
            /// <returns></returns>
            public static string GetAllShippingTypes()
            {
                StringBuilder strBuilder = new StringBuilder();
                strBuilder.Append("SELECT ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxShippingTypeTable.NAME, PmxShippingTypeTable.Columns.Name.NAME, StringValueDataSet.Columns.STRING_VALUE.Name);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxShippingTypeTable.NAME, PmxShippingTypeTable.Columns.Code.NAME);

                strBuilder.Append(" FROM ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxShippingTypeTable.NAME));

		    strBuilder.Append(" WHERE ");
		    strBuilder.Append(SqlCommandHelper.EscapeTableColumnName(PmxShippingTypeTable.NAME, PmxShippingTypeTable.Columns.Name.NAME));
		    strBuilder.Append(" NOT LIKE 'ZZ%' ");


                return strBuilder.ToString();
            }



            /// <summary>
            /// Gets all sales delivery reports.
            /// </summary>
            /// <returns></returns>
            public static string GetAllSalesDeliveryReports()
            {
                StringBuilder strBuilder = new StringBuilder();
                strBuilder.Append("SELECT ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxReportTable.NAME, PmxReportTable.Columns.Name.NAME, StringValueDataSet.Columns.STRING_VALUE.Name);

                strBuilder.Append(" FROM ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxReportTable.NAME));

                strBuilder.AppendFormat("\r\n WHERE {0} = {1}",
                    SqlCommandHelper.EscapeTableColumnName(PmxReportTable.NAME, PmxReportTable.Columns.ReportTypeCode.NAME),
                    SqlCommandHelper.FormatAndEscapeColumnValue(PmxReportTypeCodes.SALES_DELIVERY_REPORT_TYPE_CODE, CultureInfo.InvariantCulture)
                    );

                return strBuilder.ToString();
            }

            /// <summary>
            /// Gets the sales order document entry and line nums.
            /// </summary>
            /// <param name="pickListDocEntry">The pick list document entry.</param>
            /// <returns></returns>
            public static string GetSalesOrderDocEntryAndLineNums(int pickListDocEntry)
            {
                StringBuilder strBuilder = new StringBuilder();
                strBuilder.Append("SELECT DISTINCT ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListProposalLineTable.NAME, PmxPickListProposalLineTable.Columns.BaseEntry.NAME);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListProposalLineTable.NAME, PmxPickListProposalLineTable.Columns.BaseLine.NAME);

                strBuilder.Append(" FROM ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxPickListProposalLineTable.NAME));

                strBuilder.AppendFormat("\r\n INNER JOIN {0} ON {1} = {2} AND {3} = {4} AND {5} = {6}",
                    SqlCommandHelper.EscapeTableName(PmxPickListLineTable.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.BaseEntry.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxPickListProposalLineTable.NAME, PmxPickListProposalLineTable.Columns.DocEntry.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.BaseLine.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxPickListProposalLineTable.NAME, PmxPickListProposalLineTable.Columns.LineNum.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxPickListProposalLineTable.NAME, PmxPickListProposalLineTable.Columns.BaseType.NAME),
                    SqlCommandHelper.FormatAndEscapeColumnValue(SboObjectTypes.SalesOrder, CultureInfo.InvariantCulture)
                    );

                strBuilder.AppendFormat("\r\n INNER JOIN {0} ON {1} = {2}",
                    SqlCommandHelper.EscapeTableName(PmxPickListTable.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.DocEntry.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxPickListTable.NAME, PmxPickListTable.Columns.DocEntry.NAME)
                    );

                strBuilder.AppendFormat("\r\n WHERE {0} IN ( ",
                      SqlCommandHelper.EscapeTableColumnName(PmxPickListTable.NAME, PmxPickListTable.Columns.WaveKey.NAME));
                {
                    strBuilder.Append("SELECT DISTINCT ");
                    SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListTable.NAME, PmxPickListTable.Columns.WaveKey.NAME);

                    strBuilder.Append(" FROM ");
                    strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxPickListTable.NAME));

                    strBuilder.AppendFormat("\r\n WHERE {0} = {1}",
                    SqlCommandHelper.EscapeTableColumnName(PmxPickListTable.NAME, PmxPickListTable.Columns.DocEntry.NAME),
                    SqlCommandHelper.FormatAndEscapeColumnValue(pickListDocEntry, CultureInfo.InvariantCulture)
                    );
                }

                strBuilder.Append(") ");





                return strBuilder.ToString();
            }

            /// <summary>
            /// Gets the transport code.
            /// </summary>
            /// <param name="shippingType">Type of the shipping.</param>
            /// <returns></returns>
            public static string GetTransportCode(string shippingType)
            {
                StringBuilder strBuilder = new StringBuilder();
                strBuilder.Append("SELECT ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxShippingTypeTable.NAME, PmxShippingTypeTable.Columns.Code.NAME);


                strBuilder.Append(" FROM ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxShippingTypeTable.NAME));

                strBuilder.AppendFormat(" WHERE {0} = {1} ",
                    SqlCommandHelper.EscapeTableColumnName(PmxShippingTypeTable.NAME, PmxShippingTypeTable.Columns.Name.NAME),
                    SqlCommandHelper.FormatAndEscapeColumnValue(shippingType, CultureInfo.InvariantCulture));


                return strBuilder.ToString();
            }
        }

        #endregion


    }
}