//css_ref Produmex.Foundation.SlimScreen.Interfaces;
//css_ref Produmex.Foundation.Wwf.Sbo;
//css_ref Produmex.Foundation.Data.dll;
//css_ref Produmex.Foundation.Data.Sbo.dll;
//css_ref Produmex.Foundation.dll;
//css_ref Produmex.Foundation.Sbo.dll;
//css_ref Produmex.Sbo.Logex.Data.dll;



using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Data;
using System.Data.Common;
using System.Globalization;
using System.IO;
using System.Reflection;
using System.Text;
using System.Xml;
using Produmex.Foundation;
using Produmex.Foundation.Convertors;
using Produmex.Foundation.Data;
using Produmex.Foundation.Data.Sbo;
using Produmex.Foundation.Data.Sbo.BusinessObjects;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions.Tables;
using Produmex.Foundation.Data.Sbo.DataObjects;
using Produmex.Foundation.Data.Sbo.Providers;
using Produmex.Foundation.Data.Sbo.Utilities;
using Produmex.Foundation.Data.SqlClient;
using Produmex.Foundation.Diagnostics;
using Produmex.Foundation.Messages;
using Produmex.Foundation.SlimScreen;
using Produmex.Foundation.Utilities;
using Produmex.Foundation.Workflows;
using Produmex.Foundation.Workflows.Parameters;
using Produmex.Foundation.Wwf.Sbo.LocalServices;
using Produmex.Sbo.Logex.Data.BusinessObjects;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables;
using Produmex.Sbo.Logex.Data.DataObjects;
using Produmex.Sbo.Logex.Data.Providers;
using Produmex.Sbo.Logex.Korver.BusinessObjects.Definitions.Tables;
using Produmex.Foundation.SlimScreen.Interfaces;
using Produmex.Foundation.Barcode;
using Produmex.Foundation.Data.DbClient;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions.KnownDataSets;
using Produmex.Sbo.Logex.Data.ViewObjects;
using Produmex.Foundation.Collections;

// PmxWorkflowExecutionTypes.HOOK_FLOW
namespace WorkflowScript_SalesDeliveryCreatedHookScript
{
    /// <summary>
    /// This is a hook script, meant to be customized on a "per project" basis
    /// </summary>
    public class WorkflowScript_SalesDeliveryCreatedHookScript : WorkflowInstanceScriptBase
    {
        private static readonly ILog s_log = LogProvider.GetLogger(MethodInfo.GetCurrentMethod().DeclaringType);


        // Input parameters * do not change *
        public ReadOnlyBinder<CultureInfo> DefaultCultureInfo;
        public ReadOnlyBinder<PmxOseCompany> PmxOseCompany;
        public ReadOnlyBinder<DocumentRef[]> DocumentRefs;
		
		
		
		public Collection<string> ListOfExcludedBP = new Collection<string>() { "D2626" };


        public static class Definitions
        {
            /// <summary>
			/// The EXCLUDED SHIPPING TYPES
            /// </summary>
			public static Collection<int> EXCLUDED_SHIPPING_TYPES = new Collection<int>() { 7, 10, 17, 18, 24, 25 };

            public const string REPORT_PATH_LABEL = "Korver_OutboundLabelV2.rpt";

            public const string PRINT_DOCK = "VERZENDING";

			

        }

        // Output parameters
        // <none>

        // Sub-flows
        // <none>


        /// <summary>
        /// Initializes a new instance of the <see cref="WorkflowScript_SalesDeliveryCreatedHookScript"/> class.
        /// </summary>
        /// <param name="parent">The parent.</param>
        /// <param name="factory">The factory.</param>
        public WorkflowScript_SalesDeliveryCreatedHookScript(WorkflowInstanceBase parent, WorkflowInstanceFactory factory)
            : base(parent, factory)
        {
        }

        #region WorkflowInstanceScriptBase Members

		protected override void Execute()
		{
			// Parameters in scope
			Session session = GetScopeParameter("Session") as Session;
			ISboProviderService sboProviderService = GetScopeParameter("<WwfService>ISboService") as ISboProviderService;

			Message msg = null;

			string query = null;
			ProdumexError pmxError = null;
			string msgKey = null;
			string initialErrorKey = null;
			string gtin = null;
			string itemCode = null;
			double quantity = 0;

			DocumentRef[] documentRefs = DocumentRefs.Get();


			foreach (DocumentRef docRef in documentRefs)
			{
				if (docRef.DocType == SboObjectTypes.SalesDelivery)
				{


					query = BuildQuery.GetTotalColli(docRef.DocEntry, sboProviderService.GetDbTool().CultureInfo);

					DataSet ds = sboProviderService.RunView(false, null, null, query);

					if (this.GetNumberOfRows(ds) > 0)
					{
						int numberOfColli = Convertor.ConvertToInt32Nullable(ds.Tables[0].Rows[0][PmxSalesDeliveryTable.Columns.Colli.NAME]) ?? 0;
						int shippingType = Convert.ToInt32(ds.Tables[0].Rows[0][SboSalesDeliveryNoteTable.Columns.TrnspCode.NAME]);
						int docNum = Convert.ToInt32(ds.Tables[0].Rows[0][SboSalesDeliveryNoteTable.Columns.DocNum.NAME]);

						string shippingDock = Convert.ToString(ds.Tables[0].Rows[0][PmxPickListTable.Columns.DestStorLocCode.NAME]);
						string cardCode = Convert.ToString(ds.Tables[0].Rows[0][PmxSalesDeliveryNoteTable.Columns.CardCode.NAME]);

						string sboWhsCode = Convert.ToString(ds.Tables[0].Rows[0][PmxOseWarehouseTable.Columns.SboWarehouseCode.NAME]);
						string pmxWhsCode = Convert.ToString(ds.Tables[0].Rows[0][PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME]);

						#region PRINT LABEL
						
						try
						{

							

							if (ListOfExcludedBP.Contains(cardCode))
							{
								s_log.InfoFormat("No label printed for delivery {0}, because it is for card code {1}", docRef.DocEntry, cardCode);
							}
							else
							{

								if (!Definitions.EXCLUDED_SHIPPING_TYPES.Contains(shippingType))
								{
									//Print label
									s_log.InfoFormat("Print {0} times the label for delivery {1}", numberOfColli, docRef.DocEntry);

									//print label
									sboProviderService.InvokeMethodWithDbConnection<PmxOsePrinter>(false, false, null, null, new DelegateWithPmxDbConnection<PmxOsePrinter>(ExecutePrintLabel), docRef.DocEntry, session.DeviceInstanceId, numberOfColli, string.IsNullOrEmpty(shippingDock) ? Definitions.PRINT_DOCK : shippingDock);


								}
								else
								{
									s_log.InfoFormat("No label printed for delivery {0}, because it has shipping type {1}", docRef.DocEntry, shippingType);
								}
							}

						


						}
						catch (ProdumexException ex)
						{
							s_log.Error("", ex);
							msgKey = ExceptionHelper.Format(ex, this.DefaultCultureInfo);
							pmxError = ex.Error;
						}
						catch (Exception ex)
						{
							s_log.Error("", ex);
							msgKey = ExceptionHelper.Format(ex, this.DefaultCultureInfo);
						}

						if (msgKey != null)
						{
							session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
								DefaultCultureInfo.Get(), BuildParamCollection(
									"InitialErrorKey", null,
									"MessageKey", msgKey,
									"PmxError", pmxError,
									"ShowButton", true));
							WaitForMessage();
						}
						#endregion

						PmxDictionary<string, double> listOfGlassContainers = new PmxDictionary<string, double>();

					Step_AskQuestion:

						#region ASK QUESTION

						listOfGlassContainers.Clear();

						session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
						   DefaultCultureInfo.Get(), BuildParamCollection(
						   "TitleKey", "MSG_TITLE_KOR_SENT_GLASS_CONTAINER",
						   "MessageKey", "MSG_QUESTION_KOR_SENT_GLASS_CONTAINER",
						   "Parameters", new object[] { docNum }));
						msg = WaitForMessage();
						if (msg.Name.ToLower().EndsWith(".yes"))
						{
							//Add glass container
							goto Step_IdentifyGlassContainer;
						}
						else
						{
							goto Step_End;
						}

						#endregion

					Step_IdentifyGlassContainer:

						#region IDENTIFY PRODUCT

						List<string> buttons = new List<string>();

						buttons.Add("MSG_BUTTON_SELECT_GLASS_CONTAINER_KORVER");

						if (listOfGlassContainers.Count > 0)
						{
							buttons.Add("MSG_BUTTON_DONE");
						}

						session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IIdentifyGS1DataScreen),
						  DefaultCultureInfo.Get(), BuildParamCollection(
							  "InitialErrorKey", initialErrorKey,
							  "TitleKey", "MSG_TITLE_IDENTIFY_GLASS_CONTAINER_KORVER",
							  "AllowedButtons", buttons.ToArray(),
							  "ImageType", GS1ImageType.BoxImage
							  ));
						msg = WaitForMessage();

						gtin = null;
						initialErrorKey = null;
						itemCode = null;
						quantity = 0;

						if (msg.Name.EndsWith(".DataEntered"))
						{
							BarcodeContainer barcodeContainerObject = ExtractParameter<BarcodeContainer>(msg.Parameters, "data");

							gtin = BarcodeHelper.GetProductIdentification(barcodeContainerObject);

							if (string.IsNullOrEmpty(gtin))
							{
								initialErrorKey = "MSG_ERROR_NO_PRODUCT_ON_BARCODE";
								goto Step_IdentifyGlassContainer;
							}

							goto Step_SelectProduct;

						}

						if (msg.Name.EndsWith(".ButtonClicked"))
						{
							string buttonKey = ExtractParameter<string>(msg.Parameters, "buttonKey");

							if (buttonKey == "MSG_BUTTON_SELECT_GLASS_CONTAINER_KORVER")
							{
								gtin = null;
								goto Step_SelectProduct;
							}

							goto Step_CreateDelivery;
						}

						//Back requested
						goto Step_AskQuestion;


						#endregion


					Step_SelectProduct:

						#region SELECT PRODUCT

						query = BuildQuery.GetProductsFromStockOrderByName(gtin, null, null, null, false, Produmex.Sbo.Logex.Data.SboCommands.PackagingUnit.Inventory, null, false, false, sboProviderService.GetDbTool());
						DataSet dsItems = sboProviderService.RunView(false, null, null, query);

						if (GetNumberOfRows(dsItems) == 0)
						{
							initialErrorKey = "MSG_ERROR_PRODUCT_NOT_FOUND";
							goto Step_IdentifyGlassContainer;
						}

						if (GetNumberOfRows(dsItems) == 1)
						{
							itemCode = Convert.ToString(dsItems.Tables[0].Rows[0][ProductDataSet.Columns.PRODUCT_CODE.Name]);

							goto Step_EnterQuantity;

						}
						else
						{
							//Multiple items found.
							//So select it from a list

							session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectProductScreen),
								DefaultCultureInfo.Get(), BuildParamCollection(
								"InitialErrorKey", initialErrorKey,
								"TitleKey", "MSG_TITLE_SELECT_PRODUCT",
								"ProductDS", dsItems));

							msg = WaitForMessage();

							//clear data
							initialErrorKey = null;
							itemCode = null;

							if (msg.Name.EndsWith(".ProductSelected"))
							{
								itemCode = ExtractParameter<string>(msg.Parameters, "itemCode");
								goto Step_EnterQuantity;
							}
							else if (msg.Name.EndsWith(".BackRequested"))
							{
								// We go back on previous step
								goto Step_IdentifyGlassContainer;
							}

							goto Step_IdentifyGlassContainer;

						}


						#endregion

					Step_EnterQuantity:

						#region ENTER QUANTITY

						double maxQuantity = 0;

						//get maximum quantity
						sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null,
						  delegate(PmxDbConnection conn, object[] parameters)
						  {
							  PmxInventoryProvider prov = new PmxInventoryProvider(conn);
							  double? dummy;
							  maxQuantity = prov.GetStockQuantityExtended(itemCode, null, false, null, null, null, null, null, false, false, false, pmxWhsCode, out dummy);

							  return null;
						  });

						//Reduce quantity already added
						if (listOfGlassContainers.ContainsKey(itemCode))
						{
							maxQuantity -= listOfGlassContainers[itemCode];
						}

						if (maxQuantity > 0)
						{
							quantity = 1;
							goto Step_AddToList;
						}
						else
						{
							initialErrorKey = "MSG_ERROR_NOT_IN_STOCK";
							goto Step_IdentifyGlassContainer;
						}


						////ask for quantity
					//session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterQuantityScreen),
					//    DefaultCultureInfo.Get(), BuildParamCollection(
					//        "InitialErrorKey", initialErrorKey,
					//        "TitleKey", "MSG_TITLE_ENTER_QUANTITY",
					//        "InitialQuantity", 1,
					//        "NumberOfDigitsAfterDecimal", 0,
					//        "MaximumQuantity", maxQuantity,
					//        "UOM", ""
					//       ));
					//msg = WaitForMessage();

						////clear data
					//initialErrorKey = null;

						//if (msg.Name.EndsWith(".QuantityEntered"))
					//{
					//    quantity = ExtractParameter<double>(msg.Parameters, "quantity");
					//    goto Step_AddToList;
					//}

						////go back
					//goto Step_IdentifyGlassContainer;



						#endregion

					Step_AddToList:

						#region ADD TO LIST

						if (!listOfGlassContainers.ContainsKey(itemCode))
						{
							listOfGlassContainers.Add(itemCode, 0);
						}

						listOfGlassContainers[itemCode] += quantity;

						goto Step_IdentifyGlassContainer;

						#endregion

					Step_CreateDelivery:

						#region CREATE DELIVERY

						if (listOfGlassContainers.Count > 0)
						{

							try
							{

								sboProviderService.InvokeMethodWithDbConnection<object>(true, false, null, null,
								  delegate(PmxDbConnection conn, object[] parameters)
								  {
									  PmxSalesDeliveryNoteProvider delProv = new PmxSalesDeliveryNoteProvider((SboConnection)conn);
									  PmxInventoryProvider invProv = new PmxInventoryProvider(conn);

									  PmxItemAllConnectionsProvider itemProv = new PmxItemAllConnectionsProvider(conn);


									  


									  using (PmxSalesDeliveryNote delivery = delProv.GetNewBO())
									  {
										  delivery.CardCode = cardCode;
										  delivery[PmxSalesDeliveryTable.Columns.SalesDeliveryDocNum.NAME] = docNum;
  										   delivery.PaymentGroupCode = 1;

										  foreach (string itemCodeToDeliver in listOfGlassContainers.Keys)
										  {
											  PmxDictionary<string, PmxItemInfo> itemInfos = itemProv.GetCachedItemInfoInDictionary(new Collection<string>() { itemCodeToDeliver });
											  double remainingQuantity = listOfGlassContainers[itemCodeToDeliver];
											  //Get the actual stock details
											  using (PmxInventoryView rs = invProv.GetInventoryForFEFO(null, pmxWhsCode, new Collection<string>() { itemCodeToDeliver }, null, null, null, null, null, false))
											  {
												  while (!rs.EoF)
												  {

													  DocumentLineInfo delNoteLine = new DocumentLineInfo();
													  //set base info


													  delNoteLine.ItemCode = itemCodeToDeliver;
													  delNoteLine.WarehouseCode = sboWhsCode;
													  delNoteLine.Quantity = Math.Min(remainingQuantity, rs.Quantity);
													  delNoteLine.QuantityPerUom = 1;

													  delNoteLine.QualityStatusCode = rs.QualityStatusCode;
													  delNoteLine.LogisticUnitIdentKey = rs.LUID;
													  delNoteLine.StorageLocationCode = rs.StorLocCode;
													  delNoteLine.SSCC = rs.SSCC;

													  remainingQuantity -= delNoteLine.Quantity;

													  // Add it to the document
													  Produmex.Sbo.Logex.Data.Utilities.DocumentInfoHelper.AddDocumentLineInfoToBO(delNoteLine, delivery, delProv, itemInfos);


													  if (remainingQuantity <= 0)
													  {
														  break;
													  }



													  rs.MoveNext();
												  }
											  }
										  }//foreach

										  if (delivery.Lines.Count > 0)
										  {

											  session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
												   DefaultCultureInfo.Get(), BuildParamCollection(
												   "InitialErrorKey", initialErrorKey,
												   "MessageKey", "MSG_ADDING_SALES_DELIVERIES",
												   "ShowButton", false));

											  delProv.AddBO(delivery);

										  }

									  }

									  return null;
								  });
							}
							catch (Exception ex)
							{
								s_log.Error("", ex);
								if (!(ex is ProdumexException) || ((ProdumexException)ex).Error == null)
								{
									// Inform the user with the error
									session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
										DefaultCultureInfo.Get(), BuildParamCollection(
											"MessageKey", ExceptionHelper.Format(ex, this.DefaultCultureInfo),
											"NoTranslationOfMessageKey", true,
											"IsError", true,
											"ShowButton", true));
									WaitForMessage();

								}
								else
								{
									// Inform the user with the error
									session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
										DefaultCultureInfo.Get(), BuildParamCollection(
											"PmxError", ((ProdumexException)ex).Error,
											"ShowButton", true));
									WaitForMessage();
								}

								//retry?
								//ask user if he wants to retry
								session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
									DefaultCultureInfo.Get(), BuildParamCollection(
										"TitleKey", "MSG_TITLE_RETRY_UPDATE",
										"MessageKey", "MSG_QUESTION_AN_ERROR_OCCURED_WANT_RETRY"));
								msg = WaitForMessage();

								if (msg.Name.ToLower().EndsWith(".yes"))
								{
									goto Step_CreateDelivery;
								}


								goto Step_End;
							}
						}


						goto Step_End;

						#endregion

					Step_End:

						listOfGlassContainers.Clear();


					}

				}
			}


		}

        #endregion



        #region HELPERS




        /// <summary>
        /// Executes the print label.
        /// </summary>
        /// <param name="conn">The connection.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        private static PmxOsePrinter ExecutePrintLabel(PmxDbConnection conn, object[] parameters)
        {
            PmxPrintReportProvider prov = new PmxPrintReportProvider(conn);

            Collection<ReportParameter> reportParameters = new Collection<ReportParameter>();
            reportParameters.Add(new ReportParameter(Produmex.Sbo.Logex.Data.Reports.Definitions.ReportTypeSalesDeliveryNoteDefinition.Parameters.SalesDeliveryNoteDocEntry.NAME, (int)parameters[0]));

           
            return prov.PrintReport(reportParameters, Definitions.REPORT_PATH_LABEL, (int)parameters[2], (string)parameters[1], (string) parameters[3], false);
        }


        /// <summary>
        /// Gets the number of rows.
        /// </summary>
        /// <param name="ds">The ds.</param>
        /// <returns></returns>
        private int GetNumberOfRows(DataSet ds)
        {
            if (ds != null && ds.Tables.Count > 0)
            {
                return ds.Tables[0].Rows.Count;
            }
            return 0;
        }

        #endregion

        #region QUERY

        private class BuildQuery
        {


			/// <summary>
			/// Gets the name of the products from stock order by.
			/// </summary>
			/// <param name="gtin">The gtin.</param>
			/// <param name="itemCode">The item code.</param>
			/// <param name="locationCode">The location code.</param>
			/// <param name="sscc">The SSCC.</param>
			/// <param name="forceSpecific">if set to <c>true</c> [force specific].</param>
			/// <param name="packagingUnit">The packaging unit.</param>
			/// <param name="logisticCarrierLocationCode">The logistic carrier location code.</param>
			/// <param name="filterOutFrozenItems">if set to <c>true</c> [filter out frozen items].</param>
			/// <returns></returns>
			public static string GetProductsFromStockOrderByName(string gtin, string itemCode, string locationCode, string sscc, bool forceSpecific, Produmex.Sbo.Logex.Data.SboCommands.PackagingUnit packagingUnit, string logisticCarrierLocationCode, bool filterOutFrozenItems, bool filterOutLocationTrackedItemsWhenNoSSCC, DbTool dbTool)
			{
				DbQueryBuilder strBuilder = dbTool.GetQueryBuilder();
				strBuilder.Append("SELECT DISTINCT ");

				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, SboItemTable.NAME, SboItemTable.Columns.ItemCode.NAME, ProductDataSet.Columns.PRODUCT_CODE.Name);

				strBuilder.Append(", ");
				SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, SboItemTable.NAME, PmxItemTable.Columns.CustomItemDescr.NAME, ProductDataSet.Columns.PRODUCT_DESCRIPTION.Name);

				strBuilder.Append(", ");
				Produmex.Sbo.Logex.Data.SboCommands.AddBarcodeUsingPackagingUnit(strBuilder, packagingUnit, ProductDataSet.Columns.GTIN.Name);

				strBuilder.Append(", ");
				strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboItemTable.NAME));
				strBuilder.Append(".");
				strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.Columns.HasBestBeforeDate.NAME));

				strBuilder.Append(", ");
				strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboItemTable.NAME));
				strBuilder.Append(".");
				strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboItemTable.Columns.ManBtchNum.NAME));

				strBuilder.Append(", ");
				strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboItemTable.NAME));
				strBuilder.Append(".");
				strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.Columns.IsLogisticUnit.NAME));

				strBuilder.Append(", ", strBuilder.AsIdentifier(SboItemTable.NAME, SboItemTable.Columns.InventoryUom.NAME));

				strBuilder.Append(" FROM ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(SboItemTable.NAME));

				//inner join Product - inventory
				strBuilder.Append(" INNER JOIN ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));
				strBuilder.Append(" ON ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));
				strBuilder.Append(".");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.Columns.ItemCode.NAME));
				strBuilder.Append(" = ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(SboItemTable.NAME));
				strBuilder.Append(".");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(SboItemTable.Columns.ItemCode.NAME));

				strBuilder.Append(" LEFT JOIN ");
				strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemPackagingTypeTable.NAME));
				strBuilder.Append(" ON ", strBuilder.AsIdentifier(PmxItemPackagingTypeTable.NAME, PmxItemPackagingTypeTable.Columns.ItemCode.NAME));
				strBuilder.Append(" = ", strBuilder.AsIdentifier(SboItemTable.NAME, SboItemTable.Columns.ItemCode.NAME));


				string operand = " WHERE ";
				if (!string.IsNullOrEmpty(gtin))
				{
					strBuilder.Append(operand);
					//Produmex.Sbo.Logex.Data.SboCommands.AddWhereClauseSearchForBarcode(strBuilder, gtin, true);
					Produmex.Sbo.Logex.Data.SboCommands.AddWhereClauseSearchForBarcodePackagingType(strBuilder, gtin);
					operand = " AND ";
				}

				if (!string.IsNullOrEmpty(itemCode))
				{
					strBuilder.Append(operand);
					strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboItemTable.NAME));
					strBuilder.Append(".");
					strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboItemTable.Columns.ItemCode.NAME));
					strBuilder.Append(" = ");
					strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(itemCode, CultureInfo.InvariantCulture));
					operand = " AND ";
				}

				if (!string.IsNullOrEmpty(sscc))
				{
					strBuilder.Append(operand);
					strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.NAME));
					strBuilder.Append(".");
					strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.SSCC.NAME));
					strBuilder.Append(" = ");
					strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(sscc, CultureInfo.InvariantCulture));
					operand = " AND ";
				}
				else if (forceSpecific)
				{
					strBuilder.Append(operand);
					strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.NAME));
					strBuilder.Append(".");
					strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.SSCC.NAME));
					strBuilder.Append(" IS NULL ");
					operand = " AND ";

				}

				if (string.IsNullOrEmpty(sscc) && filterOutLocationTrackedItemsWhenNoSSCC)
				{
					strBuilder.Append(operand);
					strBuilder.AppendFormat(" " + strBuilder.IFNULL + "({0}, {1}) = {1} ",
						SqlCommandHelper.EscapeTableColumnName(PmxItemTable.NAME, PmxItemTable.Columns.TrackLocationSerialNumber.NAME),
						SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.N.VALUE, CultureInfo.InvariantCulture));

					operand = " AND ";
				}

				if (!string.IsNullOrEmpty(locationCode) &&
					!string.IsNullOrEmpty(logisticCarrierLocationCode))
				{
					strBuilder.Append(operand);
					strBuilder.Append("(");


					strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.NAME));
					strBuilder.Append(".");
					strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.StorLocCode.NAME));
					strBuilder.Append(" = ");
					strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(locationCode, CultureInfo.InvariantCulture));

					strBuilder.Append(" OR ");
					strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.NAME));
					strBuilder.Append(".");
					strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.StorLocCode.NAME));
					strBuilder.Append(" = ");
					strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(logisticCarrierLocationCode, CultureInfo.InvariantCulture));

					strBuilder.Append(")");

					operand = " AND ";
				}
				else
				{

					if (!string.IsNullOrEmpty(locationCode))
					{
						strBuilder.Append(operand);
						strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.NAME));
						strBuilder.Append(".");
						strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.StorLocCode.NAME));
						strBuilder.Append(" = ");
						strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(locationCode, CultureInfo.InvariantCulture));
						operand = " AND ";
					}

					if (!string.IsNullOrEmpty(logisticCarrierLocationCode))
					{
						strBuilder.Append(operand);
						strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.NAME));
						strBuilder.Append(".");
						strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.StorLocCode.NAME));
						strBuilder.Append(" = ");
						strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(logisticCarrierLocationCode, CultureInfo.InvariantCulture));
						operand = " AND ";
					}
				}


				strBuilder.Append(operand);
				strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboItemTable.NAME));
				strBuilder.Append(".");
				strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboItemTable.Columns.InventoryItem.NAME));
				strBuilder.Append(" = ");
				strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));
				operand = " AND ";

				if (filterOutFrozenItems)
				{
					Produmex.Sbo.Logex.Data.SboCommands.GetFrozenItemFilterSQLWherePart(strBuilder, operand);
				}

				strBuilder.Append(operand);
				strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboItemTable.NAME));
				strBuilder.Append(".");
				strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTableKorver.Columns.IsGlassContainer.NAME));
				strBuilder.Append(" = ");
				strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));
				operand = " AND ";


				strBuilder.Append(" ORDER BY ");
				strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboItemTable.NAME));
				strBuilder.Append(".");
				strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.Columns.CustomItemDescr.NAME));
				return strBuilder.ToString();
			}


            /// <summary>
            /// Gets the total colli.
            /// </summary>
            /// <param name="docEntry">The doc entry.</param>
            /// <param name="cultInfo">The cult info.</param>
            /// <returns></returns>
            internal static string GetTotalColli(int docEntry, CultureInfo cultInfo)
            {
                StringBuilder strBuilder = new StringBuilder();

                strBuilder.AppendFormat("SELECT {0}",
                    SqlCommandHelper.EscapeTableColumnName(SboSalesDeliveryNoteTable.NAME, PmxSalesDeliveryTable.Columns.Colli.NAME));

                strBuilder.AppendFormat(", {0}",
                    SqlCommandHelper.EscapeTableColumnName(SboSalesDeliveryNoteTable.NAME, SboSalesDeliveryNoteTable.Columns.TrnspCode.NAME));
                
				strBuilder.AppendFormat(", {0}",
					SqlCommandHelper.EscapeTableColumnName(PmxPickListTable.NAME, PmxPickListTable.Columns.DestStorLocCode.NAME));

				strBuilder.AppendFormat(", {0}",
				SqlCommandHelper.EscapeTableColumnName(SboSalesDeliveryNoteTable.NAME, SboSalesDeliveryNoteTable.Columns.CardCode.NAME));

				strBuilder.AppendFormat(", {0}",
				SqlCommandHelper.EscapeTableColumnName(SboSalesDeliveryNoteTable.NAME, SboSalesDeliveryNoteTable.Columns.DocNum.NAME));

				strBuilder.AppendFormat(", {0}",
				SqlCommandHelper.EscapeTableColumnName(PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.SboWarehouseCode.NAME));
				
				strBuilder.AppendFormat(", {0}",
				SqlCommandHelper.EscapeTableColumnName(PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME));

                strBuilder.AppendFormat("\r\nFROM {0}",
                    SqlCommandHelper.EscapeTableName(SboSalesDeliveryNoteTable.NAME));

				strBuilder.AppendFormat("\r\nLEFT JOIN {0}",
					SqlCommandHelper.EscapeTableName(PmxLinkSalesDeliveryNoteLinePickListLineTable.NAME));
				strBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxLinkSalesDeliveryNoteLinePickListLineTable.NAME, PmxLinkSalesDeliveryNoteLinePickListLineTable.Columns.SalesDeliveryNoteLineDocEntry.NAME);
				strBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(strBuilder, SboSalesDeliveryNoteTable.NAME, SboSalesDeliveryNoteTable.Columns.DocEntry.NAME);

				strBuilder.AppendFormat("\r\nLEFT JOIN {0}",
					SqlCommandHelper.EscapeTableName(PmxPickListTable.NAME));
				strBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxLinkSalesDeliveryNoteLinePickListLineTable.NAME, PmxLinkSalesDeliveryNoteLinePickListLineTable.Columns.PickListLineDocEntry.NAME);
				strBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListTable.NAME, PmxPickListTable.Columns.DocEntry.NAME);

				strBuilder.AppendFormat("\r\nLEFT JOIN {0}",
					SqlCommandHelper.EscapeTableName(PmxOseGeneralTable.NAME));
				strBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.Code.NAME);
				strBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListTable.NAME, PmxPickListTable.Columns.DestStorLocCode.NAME);

				strBuilder.AppendFormat("\r\nLEFT JOIN {0}",
					SqlCommandHelper.EscapeTableName(PmxOseWarehouseTable.NAME));
				strBuilder.Append(" ON ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME);
				strBuilder.Append(" = ");
				SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Code.NAME);




                strBuilder.AppendFormat("\r\nWHERE {0} = {1}",
                    SqlCommandHelper.EscapeTableColumnName(SboSalesDeliveryNoteTable.NAME, SboSalesDeliveryNoteTable.Columns.DocEntry.NAME),
                    SqlCommandHelper.FormatAndEscapeColumnValue(docEntry, cultInfo));


                return strBuilder.ToString();
            }


        }

        #endregion
    }
}
